@@ -101,8 +101,64 @@ static gchar *
 ews_get_filename (CamelFolder *folder, const gchar *uid, GError **error)
 {
 	CamelEwsFolder *ews_folder = CAMEL_EWS_FOLDER(folder);
+	GChecksum *sha = g_checksum_new (G_CHECKSUM_SHA256);
+	gchar *ret;
 
-	return camel_data_cache_get_filename (ews_folder->cache, "cache", uid, error);
+	g_checksum_update(sha, (guchar *)uid, strlen(uid));
+	ret = camel_data_cache_get_filename (ews_folder->cache, "cur",
+					     g_checksum_get_string (sha),
+					     error);
+	g_checksum_free (sha);
+	return ret;
+}
+
+static gint
+ews_data_cache_remove (CamelDataCache *cdc,
+		       const gchar *path,
+		       const gchar *key,
+		       GError **error)
+{
+	GChecksum *sha = g_checksum_new (G_CHECKSUM_SHA256);
+	gint ret;
+
+	g_checksum_update(sha, (guchar *)key, strlen(key));
+	ret = camel_data_cache_remove (cdc, path, g_checksum_get_string (sha),
+				       error);
+	g_checksum_free (sha);
+	return ret;
+}
+
+static CamelStream *
+ews_data_cache_get (CamelDataCache *cdc,
+		    const gchar *path,
+		    const gchar *key,
+		    GError **error)
+{
+	GChecksum *sha = g_checksum_new (G_CHECKSUM_SHA256);
+	CamelStream *ret;
+
+	g_checksum_update(sha, (guchar *)key, strlen(key));
+	ret = camel_data_cache_get (cdc, path, g_checksum_get_string (sha),
+				    error);
+	g_checksum_free (sha);
+	return ret;
+}
+
+static gchar *
+ews_data_cache_get_filename (CamelDataCache *cdc,
+			     const gchar *path,
+			     const gchar *key,
+			     GError **error)
+{
+	GChecksum *sha = g_checksum_new (G_CHECKSUM_SHA256);
+	gchar *ret;
+
+	g_checksum_update(sha, (guchar *)key, strlen(key));
+	ret = camel_data_cache_get_filename (cdc, path,
+					     g_checksum_get_string (sha),
+					     error);
+	g_checksum_free (sha);
+	return ret;
 }
 
 
@@ -116,10 +172,22 @@ camel_ews_folder_get_message_from_cache (CamelEwsFolder *ews_folder, const gchar
 	priv = ews_folder->priv;
 
 	g_static_rec_mutex_lock (&priv->cache_lock);
-	stream = camel_data_cache_get (ews_folder->cache, "cur", uid, error);
+	stream = ews_data_cache_get (ews_folder->cache, "cur", uid, error);
 	if (!stream) {
+		gchar *old_fname = camel_data_cache_get_filename (ews_folder->cache, "cur",
+								  uid, error);
+		if (!g_access (old_fname, R_OK)) {
+			gchar *new_fname = ews_data_cache_get_filename (ews_folder->cache,
+									"cur", uid, error);
+			g_rename (old_fname, new_fname);
+			g_free (new_fname);
+			stream = ews_data_cache_get (ews_folder->cache, "cur", uid, error);
+		}
+		g_free (old_fname);
+		if (!stream) {
 			g_static_rec_mutex_unlock (&priv->cache_lock);
-		return NULL;
+			return NULL;
+		}
 	}
 
 	msg = camel_mime_message_new ();
@@ -416,8 +484,8 @@ camel_ews_folder_get_message (CamelFolder *folder, const gchar *uid, gint pri, G
 		}
 	}
 
-	cache_file = camel_data_cache_get_filename  (ews_folder->cache, "cur",
-						     uid, error);
+	cache_file = ews_data_cache_get_filename (ews_folder->cache, "cur",
+						  uid, error);
 	temp = g_strrstr (cache_file, "/");
 	dir = g_strndup (cache_file, temp - cache_file);
 
@@ -666,6 +734,11 @@ msg_update_flags (ESoapMessage *msg, gpointer user_data)
 
 		e_ews_message_end_item_change (msg);
 
+		mi->info.flags = mi->info.flags & (~CAMEL_MESSAGE_FOLDER_FLAGGED);
+		mi->info.dirty = TRUE;
+
+		camel_folder_summary_touch (mi->info.summary);
+
 		camel_message_info_free (mi);
 	}
 	/* Don't think we need to free the list; we already freed every element */
@@ -727,7 +800,7 @@ ews_synchronize_sync (CamelFolder *folder, gboolean expunge, GCancellable *cance
 			/* OK, the change must have been the labels */
 			mi_list = g_slist_append (mi_list, mi);
 			mi_list_len++;
-		}			
+		}
 
 		if (mi_list_len == EWS_MAX_FETCH_COUNT) {
 			success = ews_sync_mi_flags (folder, mi_list, cancellable, error);
@@ -735,14 +808,16 @@ ews_synchronize_sync (CamelFolder *folder, gboolean expunge, GCancellable *cance
 			mi_list_len = 0;
 		}
 	}
-	
+
 	if (mi_list_len)
 		success = ews_sync_mi_flags (folder, mi_list, cancellable, error);
 
 	if (deleted_uids)
 		success = ews_delete_messages (folder, deleted_uids, FALSE, cancellable, error);
 
+	camel_folder_summary_save_to_db (folder->summary, NULL);
 	camel_folder_free_uids (folder, uids);
+
 	return success;
 }
 
@@ -1056,6 +1131,7 @@ ews_refresh_info_sync (CamelFolder *folder, GCancellable *cancellable, GError **
 		g_free (((CamelEwsSummary *) folder->summary)->sync_state);
 		((CamelEwsSummary *) folder->summary)->sync_state = sync_state;
 
+		camel_folder_summary_touch (folder->summary);
 		camel_folder_summary_save_to_db (folder->summary, NULL);
 
 	} while (!rerror && !includes_last_item);
@@ -1237,7 +1313,7 @@ ews_delete_messages (CamelFolder *folder, GSList *deleted_items, gboolean expung
 				camel_folder_summary_lock (folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
 				camel_folder_change_info_remove_uid (changes, uid);
 				camel_folder_summary_remove_uid (folder->summary, uid);
-				camel_data_cache_remove(ews_folder->cache, "cache", uid, NULL);
+				ews_data_cache_remove(ews_folder->cache, "cur", uid, NULL);
 				camel_folder_summary_unlock (folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
 				deleted_items = g_slist_next (deleted_items);
 			}
